--[[
NAME
    globe_texture
NOTES
    Demonstrates drawing global land and islands using texture.
]]

-- earth relief data

require("read_etopo")
img = read_etopo("./data/etopo60.cdf")

-- find land and island mask

nr, nc = img:size()
xyz, vec, idx = zeUtl.new("double", "double", "char")
zeMake.grid2points(xyz, 1, img)

xyz:copy(img)
xyz:setarr(2, 0)

-- make land mask transparent

img:getarr(2, vec)
img:fill(0)
vec:gt(0, idx)
vec:fill(0)
vec:setidx(idx, 1)
img:insert(2, vec)
img:setarr(0, vec)
img:setarr(1, vec)
img:setarr(2, vec)

-- graph setup

render, scene, root, point, pxyz, pclr, texture
    = zeGrf.new("render", "scene", "node", "point", "vertex", "color", "texture")

w = 256; h = 128

render:add(scene)
render:set{size = {w, h}}
scene:set{node = root, viewport = {0, 0, w, h}, color = {1, 1, 1, 0}}
root:add(point)

point:set{vertex = pxyz, vertex_color = pclr}

pxyz:add(xyz)
pclr:add(img)

-- fit the image

point:translate(-nc/2, -nr/2, 0)
root:scale(w/nc, h/nr, 1)

-- create texture

render:totexture(texture, 0)

-- the globe

node, light, line, lxyz, poly, pnor, pst, blend
    = zeGrf.new("node", "light", "line", "vertex", "polygon", "vertex", "texcoord", "blend")

w = 512; h = 512; R = 200

render:set{size = {w, h}}
scene:set{viewport = {0, 0, w, h}}
root:clear()
root:add(blend, light, texture, node)
root:scale(1, 1, 1)
node:add(poly, line)

--texture:set{env = "blend"}

light:set{position = {500, 500, 500}}

poly:set{vertex = pxyz, vertex_normal = pnor, texture_coord = pst,
        color = {0, 1, 1, 1}, type = "triangles"}

arr, arr2 = zeUtl.new("double", "double")

zeMake.sphere2(arr2, R, 4)

-- trinagles should be sorted in assending order
-- so that the blending works correctly.
-- If yor intend to view the transprent globe from
-- the southern hemisphere, then the sorting should be in
-- dessending order.

zeMake.sort(arr, arr2)

-- add triangles and normals data to the polygon object

pxyz:clear()
pxyz:add(arr)
arr:shift(3)
pnor:add(arr)
arr:shift(3)
pst:add(arr)

-- grid lines

line:set{vertex = lxyz, color = {.5, .5, .5, 1}, type = "lines"}

require("xy2sphere")

for y = -90, 90, 15 do
    for x = -180, 175, 5 do
        x2, y2, z2 = xy2sphere( x, y, R+1)
        lxyz:add(x2, y2, z2)
        x2, y2, z2 = xy2sphere(x+5, y, R+1)
        lxyz:add(x2, y2, z2)
    end
end

for x = -180, 180, 30 do
    for y = -90, 85, 5 do
        x2, y2, z2 = xy2sphere(x, y, R+1)
        lxyz:add(x2, y2, z2)
        x2, y2, z2 = xy2sphere(x, y+5, R+1)
        lxyz:add(x2, y2, z2)
    end
end


roty = 0; rotx = 0

node:rotatex(rotx)

require("animate_node")
animate_node(render, w, h, node, roty, rotx)
